{  bmp -- babel macro processor
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

uses sysutils,dos;

{ ****** paramerr ***************************************************** }
procedure paramerr(msg:ansistring);
begin
    writeln(stderr,'Usage: '+paramstr(0)+' <filename> [<Options>]');
    writeln(stderr,'Parameters: ');
    writeln(stderr,'    <filename>  Source file generated by bmp');
    writeln(stderr,'Options: ');
    writeln(stderr,'    -l          Line info pattern (same value as for bmp, default: ''{#line "%1" %2}'')');
    writeln(stderr,'    -p          Position of the line number in the compiler output, default: 1');
    writeln(stderr,'                1 means first number after file name,');
    writeln(stderr,'                2 means second number after file name, ');
    writeln(stderr,'                -1 means first number before file name, ...');
    writeln(stderr,'    -2          Output to stderr (default: stdout)');
    writeln(stderr,'    -h          Help');
    if msg<>'' then
    begin
        writeln(stderr);
        writeln(stderr,msg);
        halt(1);
    end
    else halt(0);
end;

{ ****** main ********************************************************* }
const max_lineinfos = 1024*1024;

var i,j,k,l,l1,l2 : longint;
    s,s1,s2       : shortstring;
    t             : text;
    first         : boolean;
    pout          : ^text;

    filename,lineinfopattern   : shortstring;
    linepos                    : longint;
    lip1,lip2,lip3             : shortstring;
    lio                        : boolean;  { true:filename first}


    libuf_fn                   : array[0..max_lineinfos-1] of ansistring;
    libuf_fnln                 : array[0..max_lineinfos-1] of longint;
    libuf_ln                   : array[0..max_lineinfos-1] of longint;
    libuf_pt                   : longint;

begin
    if paramcount<1 then paramerr('<file name> not defined');
    filename:=paramstr(1);
    lineinfopattern:='{#line "%1" %2}';
    linepos:=1;
    pout:=@output;

    filename:='';
    i:=1;
    while i<=paramcount do
    begin
        s:=paramstr(i);
        if (filename='') and (s[1]<>'-') then
        begin
            filename:=s;
            s:='';
        end;
        if s='-l' then
        begin
            if i>=paramcount then paramerr('String expected after -l');
            i+=1;
            lineInfoPattern:=paramstr(i);
{$ifdef WINDOWS}
            for j:=1 to length(lineInfoPattern) do
                if lineInfoPattern[j]='\' then lineInfoPattern[j]:='"';
{$endif}
        end
        else if s='-p' then
        begin
            if i>=paramcount then paramerr('String expected after -l');
            i+=1;
            val(paramstr(i),j,k);
            if k=0 then linepos:=j
            else paramerr('number expected after -p');
        end
        else if s='-h' then paramerr('')
        else if s='-2' then pout:=@stderr
        else if s<>'' then paramerr('Invalid option: '+s);
        i+=1;
    end;
    if linepos=0 then paramerr('Position of the line number (-p) must not be 0');
    if not fileexists(filename) then
    begin
        writeln(stderr,'File does no exist: ''',filename,'''');
        halt(2);
    end;

{ parse lineinfo pattern }
    i:=1;
    while (i<length(lineinfopattern)) and ((lineinfopattern[i]<>'%') or ((lineinfopattern[i+1]<>'1') and (lineinfopattern[i+1]<>'2'))) do
        i+=1;
    if (i>=length(lineinfopattern)) or (lineinfopattern[i]<>'%') then paramerr('Invalid format of line info pattern');
    lip1:=copy(lineinfopattern,1,i-1);
    lio:=lineinfopattern[i+1]='1';
    i+=2;
    j:=i;
    while (i<length(lineinfopattern)) and ((lineinfopattern[i]<>'%') or ((lineinfopattern[i+1]<>'1') and (lineinfopattern[i+1]<>'2'))) do
        i+=1;
    if (i>=length(lineinfopattern)) or (lineinfopattern[i]<>'%') then paramerr('Invalid format of line info pattern');
    if lio<>(lineinfopattern[i+1]='2') then paramerr('Invalid format of line info pattern');
    if i=j then paramerr('Invalid format of line info pattern');
    lip2:=copy(lineinfopattern,j,i-j);
    lip3:=copy(lineinfopattern,i+2,length(lineinfopattern)-i-1);
//    writeln('->',lip1,'<--->',lip2,'<--->',lip3,'<-');

{ read line infos from source file }
    libuf_fn[0]:=filename;
    libuf_fnln[0]:=1;
    libuf_ln[0]:=0;
    libuf_pt:=1;

    assign(t,filename);
    reset(t);
    l:=0;
    while not eof(t) do
    begin
        l+=1;
        readln(t,s);

        if copy(s,1,length(lip1))<>lip1 then continue;

        i:=length(lip1)+2;
        while (i+length(lip2)<=length(s)) and (copy(s,i,length(lip2))<>lip2) do
            i+=1;
        if i+length(lip2)>length(s) then continue;

        j:=length(s)-length(lip3)+1;
        while (j>i+length(lip2)) and (copy(s,j,length(lip3))<>lip3) do
            j-=1;
        if j<=i+length(lip2) then continue;

        s1:=copy(s,length(lip1)+1,i-length(lip1)-1);
        s2:=copy(s,length(lip2)+i,j-length(lip2)-i);

        if not lio then
        begin
            s:=s2;
            s2:=s1;
            s1:=s;
        end;
        val(s2,i,k);
        if k<>0 then continue;

        if libuf_pt>=max_lineinfos then
        begin
            writeln(stderr,'Line info buffer overflow');
            halt(2);
        end;

        if libuf_fn[libuf_pt-1]<>s1
        then libuf_fn[libuf_pt]:=s1
        else libuf_fn[libuf_pt]:=libuf_fn[libuf_pt-1];
        libuf_fnln[libuf_pt]:=i;
        libuf_ln[libuf_pt]:=l;
        libuf_pt+=1;
    end;
    close(t);

{ transform messages }
    first:=true;
    while not eof(input) do
    begin
        if not first then writeln(pout^, s);
        readln(s);
        first:=false;

        i:=1;
        while (i+length(filename)-1<=length(s)) and (copy(s,i,length(filename))<>filename) do
            i+=1;
        l1:=i;
        if i+length(filename)-1>length(s) then continue;

        if linepos>0 then
        begin
            j:=i+length(filename);
            k:=1;
        end
        else begin
            j:=i-1;
            k:=-1;
        end;
        for l:=0 to abs(linepos)-1 do
        begin
            i:=j;
            while (i>=1) and (i<=length(s)) and ((s[i]<'0') or (s[i]>'9')) do
                i+=k;
            j:=i;
            while (j>=1) and (j<=length(s)) and (s[j]>='0') and (s[j]<='9') do
                j+=k;
        end;

        j-=k;
        if k<0 then
        begin
            k:=i;
            i:=j;
            j:=k;
        end;
        if (i<1) or (j>length(s)) or (j<i) then continue;

        l2:=i;
        val(copy(s,i,j-i+1),k,i);
        if i>0 then continue;

        i:=0;
        while (i+1<libuf_pt) and (libuf_ln[i+1]<=k) do
            i+=1;
        k:=libuf_fnln[i] + k-libuf_ln[i]-1;

        str(k,s1);
        if (l1<l2) then
        begin
            delete(s,l2,j-l2+1);
            insert(s1,s,l2);
            delete(s,l1,length(filename));
            insert(libuf_fn[i],s,l1);
        end
        else begin
            delete(s,l1,length(filename));
            insert(libuf_fn[i],s,l1);
            delete(s,l2,j-l2+1);
            insert(s1,s,l2);
        end;
    end;
    writeln(pout^, s);
end.
