{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name add}
{@description Adds images (with variable factors)}

{$I directives.pas}
uses simage,getparam;
var par         : tparams;
    cc          : CSImg_Config;
    ifn,fs      : TAAStr;
    mi          : CSImg_MetaInfo;
    fo          : ansistring;

    fi,fj       : TSImg_FPImage;
    i,j         : longint;
    f           : array of double;
    g           : array[0..1] of double;
    sdb         : boolean;

begin
    with par do
    begin
        init;
        paramusage:='<infiles>';
        ifn.size:=0;    defparam('<infiles>','Input images (can be specified multiple times)',ifn);
        fs.size:=0;     defparam('-f','Factors (can be specified multiple times, default: 1)',fs);
        fo:='';         defparam('-o','Output image (default: stdout)',fo);             deffmt('-o','<outfile>');
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('add: ifn.size'+int2str(ifn.size)+' fs.size='+int2str(fs.size));

    { validate parameters }
    if ifn.size<2 then cc.error('add: At least two input images required');
    setlength(f,ifn.size);
    for i:=0 to ifn.size-1 do
        if i>=fs.size then f[i]:=1
        else if not val2(fs.aa[i],f[i]) then cc.error('add: Number expected after -f, got "'+fs.aa[i]+'"');

    { 1st input }
    mi:=CSImg_MetaInfo.create;
    cc.Info('add: Reading 1st. input image '+ifn.aa[0]);
    SImg_ReadImageFile(cc, ifn.aa[0], fi, mi);
    sdb:=((fi.nc and 1) = 0) and mi.get('ALGNB22',g[0]);
    if sdb then cc.info('add: Detected stack database');
    if abs(f[0]-1)>1e-6 then
    begin
        g[0]:=f[0]; g[1]:=f[0];
        if sdb and (abs(f[0])>1e-10) then
        begin
            g[0]:=1/f[0];
            g[1]:=1/sqr(f[0]);
        end;
        for i:=0 to fi.size-1 do
            fi.pt[i]*=g[i and 1];
    end;

    { add }
    for j:=1 to ifn.size-1 do
    begin
        cc.Info('add: Reading and adding '+ifn.aa[j]);
        SImg_ReadImageFile(cc, ifn.aa[j], fj);
        if fj.size<>fi.size then cc.error('add: Invalid dimensions of '+ifn.aa[j]);
        g[0]:=f[j]; g[1]:=f[j];
        if sdb and (abs(f[j])>1e-10) then
        begin
            g[0]:=1/f[j];
            g[1]:=1/sqr(f[j]);
        end;
        for i:=0 to fi.size-1 do
            fi.pt[i]+=fj.pt[i]*g[i and 1];
    end;
    SImg_Free(cc,fj);

    { output }
    mi.deleteRaw;
    mi.delete('NOISEFLR');
    cc.Info('add: Writing output image '+fo);
    SImg_WriteImageFile(cc,fo,fi,mi);
    mi.destroy;
    SImg_Free(cc,fi);
    cc.destroy;
end.
