{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name approx}
{@description Approximation by polynomials.
{@longdescription
Approximation by polynomials. Orthogonalization allows to weight polynomials independently
using parameters -r*. Using parameter -i additional iterations are performed where
values are capped to previous approximation. This allows to find an approximation
for the background. parameter -dm is given the difference between input If parameter
-dm is given the difference between input and approximation is written, otherwise the
approximation.}

{$I directives.pas}
uses simage,getparam;
var par                  : tparams;
    cc                   : CSImg_Config;
    fi                   : TSImg_FPImage;
    ifn,ofn              : ansistring;
    rr,r1,r1b            : double;
    frad                 : double;
    it,n                 : longint;
    dm                   : boolean;
    mi                   : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
                      paramusage:='[<infile> [<outfile>]]';
        ifn:='';      defparam('<infile>','Source image (default: stdin)',ifn);
        ofn:='';      defparam('<outfile>','Destination image (default: stdout)',ofn);
                      defparam('-h','Help',helpf);
        rr:=1.0;      defparam('-rr','Factor for radial polynomials (x^2+y^2)^i, default 1.0', rr);
        r1:=1.0;      defparam('-r1','Factor for 1st order polynomials (x and y), default 1.0', r1);
        r1b:=0.0;     defparam('-r1b','Factor for 1st order bi-polynomial (x*y), default 0.0', r1b);
        n:=0;         defparam('-n','Highest order radial polynomials ( (x^2+y^2))^i ), default 0', n);
        it:=0;        defparam('-i','Background approximation iterations (cap values to previous approximation), default:0',it);
        frad:=0;      defparam('-m','Radius of median pre-filter (default:0)',frad);    frad:=0;
        dm:=false;    defparam('-d','Output difference instead of approximation',dm);
                      defparam('-o','Destination image',ofn); deffmt('-o','<outfile>');
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('approx: values: source='+ifn+' dest='+ofn+' rr='+real2str(rr)+' r1='+real2str(r1)+' r1b='+real2str(r1b)+' n='+int2str(n)+' it='+int2str(it)+' dm='+boolean2str(dm)+' frad='+real2str(frad));

    mi:=CSImg_MetaInfo.create();
    fi.pt:=nil;
    cc.Info('approx: Reading input image '+ifn);
    SImg_ReadImageFile(cc,ifn,fi,mi);
    mi.delete('DATAMIN');
    mi.delete('DATAMAX');

    cc.Info('approx: Approximating using ' +int2str(cc.threads) + ' threads');
    SImg_Approx(cc,fi, rr,r1,r1b, n, it,dm, frad);

    cc.Info('approx: Writing output image '+ofn);
    SImg_WriteImageFile(cc,ofn,fi,mi);

    SImg_Free(cc,fi);
    cc.destroy;
end.

