{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name drcompress2}
{@description Dynamic range compression using non-linear high-pass filter and regularization method 2}

{$I directives.pas}
uses simage,getparam,sysutils;
var par         : tparams;
    cc          : CSImg_Config;
    img         : TSImg_FPImage;
    mask        : TSImg_RDImage;
    ifn,ofn,mfn : ansistring;
    l,q,r,s,c   : double;
    mi          : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        ifn:='';        defparam('<infile>','Source image (default: stdin)',ifn);
        ofn:='';        defparam('<outfile>','Destination image (default: stdout)',ofn);
        mfn:='';        defparam('-mask','Mask image calculated by getmask', mfn);
        l:=0.05;        defparam('-l','Black level threshold as quantile or <0 or disable it.. Default: 5%',l);
        q:=0.9;         defparam('-q','High level quantile at which background will be reduced to 50%. Default: 90%',q);
        r:=0.3;         defparam('-r','Dynamic range compression limit. Valid: 0..1. Default: 0.3',r);
        s:=2;           defparam('-s','Minimum detail size to block (default: 2)',s);
        c:=1;           defparam('-c','Saturation preservation. Valid: 0..1. Default: 1',c);
                        defparam('-o','Destination image',ofn); deffmt('-o','<outfile>');
                        defparam('-h','Help',helpf);
        run;
        done;
    end;

    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug('drcompress2: values: source='+ifn+' dest='+ofn+' mask='+mfn+' l='+real2str(l)+' q='+real2str(q)+' r'+real2str(r)+' s='+real2str(s)+' c='+real2str(c));

    img.pt:=nil;
    mi:=CSImg_MetaInfo.create();
    cc.Info('drcompress2: Reading input image '+ifn);
    SImg_ReadImageFile(cc,ifn,img,mi);
    mi.deleteRaw;

    mask.pt:=nil;
    if mfn<>'' then
    begin
        cc.Info('drcompress2: Reading mask image '+mfn);
        SImg_ReadImageFile(cc,mfn,mask);
    end;

    cc.Info('drcompress2: High-pass filtering using '+int2str(cc.threads)+' threads');
    SImg_DRCompress2(cc, img,mask, l,q, r, s, c);

    cc.Info('drcompress2: Writing output image '+ofn);
    SImg_WriteImageFile(cc,ofn,img,mi);

    SImg_Free(cc,img);
    if mask.pt<>nil then SImg_Free(cc,mask);
    mi.destroy;
    cc.destroy;
end.
