{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name denoise}
{@description Denoising.}

{$I directives.pas}
uses simage,getparam;
var par              : tparams;
    cc               : CSImg_Config;
    img              : TSImg_FPImage;
    fi,fo            : ansistring;
    rad,sp,thr,fwhm  : double;
    mi               : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';      defparam('<infile>','Source image (default: stdin)',fi);
        fo:='';      defparam('<outfile>','Destination image (default: stdout)',fo);
        rad:=0.5;    defparam('-r','Filter radius parameter. Valid: >=0, Recommended: 0..1, Default: 0.5. ', rad);
        sp:=1;       defparam('-s','Structure preservation. Valid: 0..1, Default and recommended: 1.', sp);
        thr:=0.95;   defparam('-t','Filtering threshold as quantile. Default: 95%. Can be increased if image contains no stars. Reduce it is stars become blurry.', thr);
        fwhm:=-1;    defparam('-w','Input FWHM. Default: auto-detect',fwhm);
                     defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
                     defparam('-h','Help',helpf);
        run;
        done;
    end;
    cc:=CSImg_Config.create(SImg_Config_Default);
    cc.debug(cc.name+': fi='+fi+' fo='+fo+' rad='+real2str(rad)+' sp='+real2str(sp)+' thr='+real2str(thr)+' fwhm='+real2str(fwhm,-1));

    mi:=CSImg_MetaInfo.create;
    cc.Info('denoise: Reading input image '+fi);
    SImg_ReadImageFile(cc,fi,img,mi);

    if fwhm<0.1 then
    begin
        if mi.get('FWHMPIX',fwhm) then cc.Info(cc.name+': Read PSF FWHM width from input image: ' +real2str(fwhm,-1))
        else begin
            fwhm:=SImg_GetSharpnessFWHM(cc,img);
            cc.Info(cc.name+': Autodetected input FWHM: ' +real2str(fwhm,-1));
        end;
    end;

    cc.Info(cc.name+': Denoising using up to ' +int2str(cc.threads) + ' threads');
    SImg_Denoise(cc, img, fwhm, rad,sp,thr);

    cc.Info(cc.name+': Writing output file '+fo);
    SImg_WriteImageFile(cc,fo,img,mi);

    SImg_Free(cc,img);
    cc.destroy;
end.
