{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name histcut}
{@description Crop and scale brightness with linear or non-linear tonal curve }

{$I directives.pas}
uses simage,getparam,sysutils;
var par        : tparams;
    cc         : CSImg_Config;
    img        : TSImg_FPImage;
    s,nc       : boolean;
    fi,fo      : string;
    l,u,mq,mv  : double;
    sp,gam,bet : double;
    gm,mm      : boolean;
    la,ua,ma   : boolean;
    mi         : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';     defparam('<infile>','Source image (default: stdin)',fi);
        fo:='';     defparam('<outfile>','Destination image (default: stdout)',fo);
                    defparam('-h','Help',helpf);

        l:=0.0;     defparam('-l','Lower value (default: 0)',l);
        la:=false;  defparam('-la','Interpret lower value as absolute value (interpreted as quantile by default)',la);

        u:=1.0;     defparam('-u','Upper value (default: 100%)',u);
        ua:=false;  defparam('-ua','Interpret upper value as absolute value (interpreted as quantile by default)',ua);

        mq:=-1e300; defparam('-mq','Mid quantile',mq);
        ma:=false;  defparam('-ma','Interpret half value as absolute value (interpreted as quantile by default)',ma);
        mv:=0.5;    defparam('-mv','Value for half quantile (default: 0.5)',mv);

        gam:=0;     defparam('-g','Gamma (default: 0)',gam);
        bet:=0;     defparam('-b','Beta (default: 0)',bet);
        sp:=0;      defparam('-s','Saturation preservation (0..1, default: 0)',sp);
        s:=false;   defparam('-st','Stretch values to [0,1]',s);
        nc:=false;  defparam('-nc','Don''t cut (only useful with -st)',nc);
                    defparam('-o','Destination image',fo); deffmt('-o','<outfile>');
        run;

        cc:=CSImg_Config.create(SImg_Config_Default);
        cc.debug('histcut: values: source='+fi+' dest='+fo+
            ' l='+real2str(l)+' u='+real2str(u)+' mq='+real2str(mq)+' mv='+real2str(mv)+
            ' gam='+real2str(gam)+' bet='+real2str(bet)+' sp='+real2str(sp)+' c='+boolean2str(not nc)+' s='+boolean2str(s)+
            ' la='+boolean2str(la)+' ma='+boolean2str(ma)+' ua='+boolean2str(ua));

        gm:=vardef(@gam) or vardef(@bet);
        mm:=(vardef(@mq) or vardef(@mv));
        if vardef(@bet) and mm then cc.error('histcut: -b and -mq/-mv can''t be defined together');
        if nc and (gm or mm) then cc.warning('histcut: -nc only works in linear mode, neither -b, -g, -mq and -mv must not be set');

        if not vardef(@mq) then mq:=abs(l)+abs(u)+1e100;

        mi:=CSImg_MetaInfo.create();
        cc.Info('histcut: Reading input image '+fi);
        SImg_ReadImageFile(cc,fi,img);
        mi.deleteRaw;

        cc.Info('histcut: Histogram transformation');
        if mm then
        begin
            if vardef(@gam) then SImg_HistCut(cc, img, l,mq,u,mv, gam, sp, s, la,ma,ua)
            else SImg_HistCut(cc, img, l,mq,u,mv, sp, s, la,ma,ua)
        end
        else begin
            if gm then SImg_HistCutGamma(cc, img, l,u, gam,bet, sp, s, la,ua)
            else SImg_HistCut(cc, img, l,u, not nc,s, la,ua);
        end;

        cc.Info('histcut: Write output image '+fo);
        SImg_WriteImageFile(cc,fo,img,mi);
        SImg_Free(cc,img);

        done;
    end;

    cc.destroy;
end.
