{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name resize}
{@description Resizes an image.}

{$I directives.pas}
uses simage,getparam;
var par            : tparams;
    cc             : CSImg_Config;
    img            : TSImg_FPImage;
    fi,fo          : ansistring;
    nx,ny,xd,yd    : longint;
    usx,usy,us     : double;
    dsx,dsy,ds     : double;
    quant          : double;
    mi             : CSImg_MetaInfo;

begin
    with par do
    begin
        init;
        paramusage:='[<infile> [<outfile>]]';
        fi:='';         defparam('<infile>','Input image (default: stdin)',fi);
        fo:='';         defparam('<outfile>','Output image (default: stdout)',fo);
                        defparam('-h','Help',helpf);
        nx:=0;          defparam('-x','x size',nx);
        ny:=0;          defparam('-y','y size',ny);
                        defparam('-*','Upscale by that factor',us);
                        defparam('-/','Downscale by that factor',ds);
                        defparam('-x*','Upscale X by that factor',usx);
                        defparam('-y*','Upscale y by that factor',usy);
        dsx:=1;         defparam('-x/','Downscale X by that factor',dsx);
        dsy:=1;         defparam('-y/','Downscale y by that factor',dsy);
        quant:=-1;      defparam('-q','Quantile for down-scaling filter or <0 for mean filter or >1 for RMS filter. Default: -1',quant);
                        defparam('-o','Output image',fo); deffmt('-o','<outfile>');
        run;
        xd:=0; yd:=0;   { how often are scaling factors dsx / dsy defined }
        if vardef(@us) then
        begin
            dsx:=1/us;  xd+=1;
            dsy:=1/us;  yd+=1;
        end;
        if vardef(@usx) then
        begin
            dsx:=1/usx; xd+=1;
        end;
        if vardef(@usy) then
        begin
            dsy:=1/usy; yd+=1;
        end;
        if vardef(@ds) then
        begin
            dsx:=ds;    xd+=1;
            dsy:=ds;    yd+=1;
        end;
        if vardef(@dsx) then xd+=1;
        if vardef(@dsy) then yd+=1;
        if (xd>1) or ((xd>0) and (nx<>0)) then paramerr('X size defined by more than one parameters');
        if (yd>1) or ((yd>0) and (ny<>0)) then paramerr('Y size defined by more than one parameters');
        done;
    end;

    cc:=CSImg_Config.create(SImg_Config_Default);

    mi:=CSImg_MetaInfo.create;
    cc.info('resize: Reading input image '+fi);
    img.pt:=nil;
    SImg_ReadImageFile(cc,fi,img,mi);
    mi.delete('BAYERPAT');

    if (nx=0) and (xd=0) and (ny<>0) then dsx:=img.ny/ny;
    if (ny=0) and (yd=0) and (nx<>0) then dsy:=img.nx/nx;
    if nx=0 then nx:=round(img.nx/dsx)
    else dsx:=img.nx/nx;
    if ny=0 then ny:=round(img.ny/dsy)
    else dsy:=img.ny/ny;

    cc.debug('resize:source= '+fi+' dest='+fo+' newx='+int2str(nx)+' newy='+int2str(ny)+' qunat='+real2str(quant));

    cc.info('resize: Resizing image from '+int2str(img.nx)+'x'+int2str(img.ny)+' to '+int2str(nx)+'x'+int2str(ny)+' using '+int2str(cc.threads)+' threads');
    if (abs(dsx)>0.9999) or (abs(dsy)>0.9999) then SImg_Resize(cc, img,img, dsx,dsy, quant)
    else SImg_Upscale(cc,img,img,nx,ny);

    cc.info('resize: Writing output image '+fo);
    mi.delete('FWHMPIX');
    SImg_WriteImageFile(cc,fo,img,mi);

    SImg_Free(cc,img);
    cc.destroy;
end.

