{  simg -- command line applications for deterministic image processing
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{@name sdbfwhm}
{@description Calculate FWHM limits for resolution optimization.}

{$I directives.pas}
uses simage,getparam,sysutils;

var par         : tparams;
    cc          : CSImg_Config;
    ifn,ofn     : ansistring;
    ri          : TSImg_RDImage;
    fi          : TSImg_FPImage;
    res         : ^TSImg_AlignmentParams;
    mi          : CSImg_MetaInfo;
    si          : TSImg_SDBInfo;
    i,n         : longint;
    ffn,afn     : TAAStr;

begin
    cc:=CSImg_Config.create(SImg_Config_Default);
    with par do
    begin
        init;
        paramusage:='[<stack database> [<output image>]]';
        ifn:='';        defparam('<stack database>','Stack database, only header is read (default: stdin)',ifn);
        ofn:='';        defparam('<output image>','Output image (default: stdout)',ofn);
        ffn.size:=0;    defparam('-f','FWHM map, can be specified multiple times',ffn); deffmt('-f','<image>');
        afn.size:=0;    defparam('-a','Alignment data, can be specified multiple times',afn);   deffmt('-a','<image>');
                        defparam('-o','Output image ',ofn);                                     deffmt('-o','<outfile>');
                        defparam('-h','Help',helpf);
        run;
        done;
    end;
    cc.debug('sdbfwhm: ifn='+ifn+'  ofn='+ofn+'  ffn.size='+int2str(ffn.size)+'  afn.size='+int2str(afn.size));
    if ffn.size<>afn.size then cc.error('sdbfwhm: Number of given FWHM maps and number of given align parameters must be equal');

    cc.Info('sdbfwhm: Reading stack database info from '+ifn);
    ri.pt:=nil;                 // used dimensions only
    mi:=CSImg_MetaInfo.create;
    SImg_RHeader(cc, ifn, ri, mi);
    if ri.nc and 1<>0 then cc.error('sdbfwhm: Invalid stack database: number of colors must be even');
    if ri.nc<4 then cc.error('sdbfwhm: Number of FWHM ranges in stack database must be at least 2');

    n:=ri.nc div 2;
    SImg_StackDB_GetInfo(cc, si, mi, ri.nx,ri.ny);
    if afn.size<n then cc.error('sdbfwhm: Number of given FWHM maps must be at least '+int2str(n));

    { alignment data }
    cc.Info('sdbfwhm: Reading '+int2str(afn.size)+' alignment data sets');
    getmem(res,sizeof(TSImg_AlignmentParams)*afn.size);
    for i:=0 to afn.size-1 do
    begin
        mi.reset;
        SImg_RHeader(cc,afn.aa[i],mi);
        if not SImg_ReadAlignmentParams(cc,res[i],mi) then cc.error(cc.name+': '+afn.aa[i]+': Align parameters not set');
    end;

    { fwhm ranges }
    fi.pt:=nil;
    SImg_StackDB_FWHMLimits(cc, si, ffn.aa[0..ffn.size-1], res[0..afn.size-1], fi, n);

    cc.info('sdbfwhm: Writing output image '+ofn);
    SImg_WriteImageFile(cc,ofn,fi);

    freemem(res,sizeof(TSImg_AlignmentParams)*afn.size);
    SImg_Free(cc,fi);
    mi.destroy;
    cc.destroy;
end.
