{  simg -- image processing library
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{*************************************************************************}
{******* SImg_ReadConfigVaraible *****************************************}
{*************************************************************************}
function SImg_ReadConfigVariable(var v:ansistring; vn:pchar):boolean;
var pc : ansistring;
begin
    pc:=getEnvironmentVariable(vn);
    result:=pc<>'';
    if result then v:=pc;
end;

function SImg_ReadConfigVariable(var v:double; vn:pchar):boolean;
var pc : ansistring;
    r  : double;
    i  : longint;
begin
    pc:=getEnvironmentVariable(vn);
    result:=pc<>'';
    if result then
    begin
        val(pc,r,i);
        if i=0 then v:=r
        else result:=false;
    end;
end;

function SImg_ReadConfigVariable(var v:longint; vn:pchar):boolean;
var pc  : ansistring;
    i,r : longint;
begin
    pc:=getEnvironmentVariable(vn);
    result:=pc<>'';
    if result then
    begin
        val(pc,r,i);
        if i=0 then v:=r
        else result:=false;
   end;
end;

function SImg_ReadConfigVariable(var v:boolean; vn:pchar):boolean;
var pc : ansistring;
begin
    pc:=getEnvironmentVariable(vn);
    result:=pc<>'';
    if result then
    begin
        v:=(pc<>'0') and (pc<>'false');
    end;
end;

{*************************************************************************}
{******* CSImg_Config ****************************************************}
{*************************************************************************}
{******* create **********************************************************}
constructor CSImg_Config.create(cc:CSImg_Config);
var i   : longint;
begin
    if cc=nil then prev:=SImg_Config_default
    else prev:=cc;

    { messages }
    MessageProcSet:=false;
    MessageProc:=nil;

    if prev=nil then
    begin
        { todo: read configuration file /etc/simg.conf }
        SImg_Default_Config_initvals;   { default settings from configure.inc }
        SImg_ReadConfigVariable(threads,'SImg_threads');
        SImg_ReadConfigVariable(pnmLE,'SImg_pnmLE');
        SImg_ReadConfigVariable(forceLE,'SImg_forceLE');
        SImg_ReadConfigVariable(forceBE,'SImg_forceBE');
        SImg_ReadConfigVariable(logFileNum,'SImg_logFileNum');
        if threads<1 then threads:=1;
        if threads>SImg_MaxThreads then threads:=SImg_MaxThreads;
        cropComp:=true;
        { draw settings }
        for i:=0 to maxcnsize-1 do
        begin
            drawcolor[i]:=1;
            backcolor[i]:=0;
        end;
        textVAlign:=1e-6;
        textHAlign:=1e-6;
        textBack:=false;
     end
     else begin
        { messages }
        EnableDebug    := prev.EnableDebug;
        EnableInfos    := prev.EnableInfos;
        EnableHints    := prev.EnableHints;
        EnableWarnings := prev.EnableWarnings;
        EnableErrors   := prev.EnableErrors;
        { other }
        pnmLE := prev.pnmLE;
        forceLE := prev.forceLE;
        forceBE := prev.forceBE;
        cropComp:= prev.cropComp;
        { threads }
        threads:=prev.threads;
        { program name }
        name:=prev.name;
        { draw settings }
        move(prev.drawcolor, drawcolor, sizeof(drawcolor));
        move(prev.backcolor, drawcolor, sizeof(backcolor));
        textVAlign:=prev.textVAlign;
        textHAlign:=prev.textHAlign;
        textBack:=prev.textBack;
     end;

end;
//udefine[SImg_Default_Config_initvals;]


{******* Messages ********************************************************}
{******* Info ************************************************************}
procedure CSImg_Config.Debug(const s:ansistring);
begin
    PrintMessage(-4,s,EnableDebug);
end;

{******* Info ************************************************************}
procedure CSImg_Config.Info(const s:ansistring);
begin
    PrintMessage(-3,s,EnableInfos);
end;

{******* Hint ************************************************************}
procedure CSImg_Config.Hint(const s:ansistring);
begin
    PrintMessage(-2,s,EnableHints);
end;

{******* Warning *********************************************************}
procedure CSImg_Config.Warning(const s:ansistring);
begin
    PrintMessage(-1,s,EnableWarnings);
end;

{******* Error ***********************************************************}
procedure CSImg_Config.Error(const s:ansistring);
begin
    PrintMessage(0,s,EnableErrors);
    raise CSImg_ErrorException.create(s);
end;

{******* FatError ********************************************************}
procedure CSImg_Config.FatError(const s:ansistring);
begin
    PrintMessage(1,s,true);
    SImg_Halt(2);
end;

{******* PrintMessage ****************************************************}
procedure CSImg_Config.PrintMessage(t:longint; const mes:ansistring; print:boolean);
begin
    if MessageProcSet then
    begin
         if MessageProc<>nil then MessageProc(t,mes,print);
    end
    else begin
        if prev<>nil then prev.PrintMessage(t,mes,print);
    end;
end;

{******* SetMessageProc **************************************************}
procedure CSImg_Config.SetMessageProc(mp:TSImg_PrintMessageProc);
begin
    MessageProcSet:=true;
    MessageProc:=mp;
end;


{*************************************************************************}
{******* CSImg_Config ****************************************************}
{*************************************************************************}
procedure SImg_InitConfig;
begin
    SImg_Config_default:=nil;
    SImg_Config_default:=CSImg_Config.create(nil);
    SImg_InitMessages;
    SImg_Config_Default.SetMessageProc(@SImg_StdPrintMessage);
end;


{*************************************************************************}
{******* CSImg_Config ****************************************************}
{*************************************************************************}
procedure SImg_DoneConfig;
begin
    SImg_Config_default.destroy;
    SImg_Config_default:=nil;
end;
