{  simg -- image processing library
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{*************************************************************************}
{******* Semaphores stuff ************************************************}
{*************************************************************************}
const SImg_Sem_Count=3;

      SImg_Sem_Mem = 0;
      SImg_Sem_Img = 1;
      SImg_Sem_Msg = 2;

      SImg_Sem_Names : array[0..SImg_Sem_Count-1] of string = ('Memory', 'Image', 'Message' );

const TSImg_Sem_wtidNum = 32;
type TSImg_Semaphore = record
       val    : boolean;
       tid    : longint;
       wtid   : array[0..TSImg_Sem_wtidNum-1] of longint;
       wtime  : array[0..TSImg_Sem_wtidNum-1] of longint;
       end;

var SImg_Sem_List : array[0..SImg_Sem_Count-1] of TSImg_Semaphore;

{******* SImg_Sem_create *************************************************}
procedure SImg_Sem_create;
var i,j   : longint;
begin
    for i:=0 to SImg_Sem_Count-1 do
    begin
        SImg_Sem_List[i].val:=false;
        SImg_Sem_List[i].tid:=0;
        for j:=0 to TSImg_Sem_wtidNum-1 do
        begin
            SImg_Sem_List[i].wtid[j]:=0;
            SImg_Sem_List[i].wtime[j]:=1000000;
        end;
    end;
end;


{******* SImg_Sem_Lock ***************************************************}
procedure SImg_Sem_Lock(num:longint);
var p,i,j,k,t  : longint;
begin
    if (num<0) or (num>=SImg_Sem_count) then
    begin
        SImg_Warning('SImg_Sem_Lock: Invalid semaphore: '+int2str(num));
        exit;
    end;
    p:=gettid;
    t:=SImg_Sem_Timeout*100;
    repeat
        if (SImg_Sem_List[num].val) and (t>0) then
        begin
            j:=-1;
            for i:=0 to TSImg_Sem_wtidNum-1 do
            begin
                k:=SImg_Sem_List[num].wtid[i];
                if (k=p) or ((k=0) and (j<0)) then j:=i;
            end;
            if j>=0 then
            begin
                SImg_Sem_List[num].wtid[j]:=p;
                SImg_Sem_List[num].wtime[j]:=t;
            end;
            delay(10);
            t-=1;
        end
        else begin
            if t<=0 then SImg_Warning('SImg_Sem_Lock('+SImg_Sem_Names[num]+'): Timeout');
            SImg_Sem_List[num].val:=true;
            SImg_Sem_List[num].tid:=p;
            delay(0);
        end;
    until SImg_Sem_List[num].val and (SImg_Sem_List[num].tid=p);

    for i:=0 to TSImg_Sem_wtidNum-1 do
        if SImg_Sem_List[num].wtid[i]=p then
        begin
            SImg_Sem_List[num].wtid[i]:=0;
            SImg_Sem_List[num].wtime[i]:=1000000;
        end;
end;

{******* SImg_Sem_TryLock ************************************************}
function SImg_Sem_TryLock(num:longint):boolean; { true if locked / on success }
var p  : longint;
begin
    result:=false;
    if (num<0) or (num>=SImg_Sem_count) then
    begin
        SImg_Error('SImg_Sem_TryLock: Invalid semaphore: '+int2str(num));
        exit;
    end;
    p:=gettid;
    if not SImg_Sem_List[num].val then
    begin
        SImg_Sem_List[num].val:=true;
        SImg_Sem_List[num].tid:=p;
        delay(0);
    end;
    result:=SImg_Sem_List[num].val and (SImg_Sem_List[num].tid=p);
end;

{******* SImg_Sem_UnLock *************************************************}
procedure SImg_Sem_UnLock(num:longint);
var i,j,k  : longint;
begin
    if (num<0) or (num>=SImg_Sem_count) then
    begin
        SImg_Error('SImg_Sem_Unlock: Invalid semaphore: '+int2str(num));
        exit;
    end;
    SImg_Sem_List[num].tid:=0;
    SImg_Sem_List[num].val:=false;
    j:=-1;  k:=1000000;
    for i:=0 to TSImg_Sem_wtidNum-1 do
        if (SImg_Sem_List[num].wtid[i]<>0) and ((j=-1) or (SImg_Sem_List[num].wtime[i]<k)) then j:=i;
    if j>=0 then SImg_WakeUp(SImg_Sem_List[num].wtid[j]);
end;

{******* SImg_Sem_Reset **************************************************}
function SImg_Sem_Reset:boolean;  { true if semaphore changed }
var i,j  : longint;
begin
    result:=false;
    for i:=0 to SImg_Sem_Count-1 do
    begin
        if SImg_Sem_List[i].val then
        begin
            SImg_Warning('SImg_Sem_Reset: Signal occurred while '+SImg_Sem_Names[i]+' semaphore locked');
            SImg_Sem_List[i].val:=false;
            SImg_Sem_List[i].tid:=0;
            for j:=0 to TSImg_Sem_wtidNum-1 do
            begin
                SImg_Sem_List[i].wtid[j]:=0;
                SImg_Sem_List[i].wtime[j]:=1000000;
            end;
            result:=true;
        end;
    end;
end;


{*************************************************************************}
{******* SImg_WakeUp *****************************************************}
{*************************************************************************}
procedure SImg_SIGUSR1(sig:longint); cdecl;
begin
end;

procedure SImg_WakeUp(tid:longint);
begin
    if tid<>0 then fpKill(tid,SIGUSR1);
end;


{*************************************************************************}
{******* SImg_Thread *****************************************************}
{*************************************************************************}
procedure CSImg_Thread.Wakeup;  { wakeup process with TID p }
begin
    fpKill(Handle,SIGUSR1);
end;


{*************************************************************************}
{******* CSImg_AutoThread ************************************************}
{*************************************************************************}
constructor CSImg_AutoThread.create(ow:boolean);
begin
    isrunning:=ow;
    oneway:=ow;
    freeonterminate:=false;
    isrunning2:=true;
    inherited create(false);
end;

destructor CSImg_AutoThread.destroy;
begin
    oneway:=true;
    while isrunning2 do
        delay(1);
    inherited destroy;
end;

procedure CSImg_AutoThread.wait;
begin
    while isrunning do
        delay(1);
end;


//define[threadprocedure ][(#1);][
procedure #0(#1); forward;

type C#0 = class(CSImg_AutoThread)
//define[:][//noexpand[!]_v:!]
         #1;
//uadefine[:]
         constructor create(#1);
         constructor create;
         procedure setwork(#1);
         procedure execute; override;
         end;
    P#0 = ^C#0;

constructor C#0.create(#1);
begin
//define[ ][:##1;][##0_v:=##0;  ]
 #1;
//udefine[ ]
    inherited create(true);
end;

constructor C#0.create;
begin
    inherited create(false);
end;

procedure C#0.setwork(#1);
begin
//define[ ][:##1;][##0_v:=##0;  ]
 #1;
//udefine[ ]
    isrunning:=true;
end;

procedure C#0.execute;
begin
    repeat
        if not oneway then delay(1);
        if isrunning then
        begin
//define[:][;][_v]
//define[ ][,]
#0(#1;);
//udefine[:]
//udefine[ ]
            isrunning:=false;
        end;
    until oneway;
    isrunning2:=false;
end;

procedure #0(#1);]

//define[threadvars;][
var AutoThreads : array[0..SImg_maxThreads] of CSImg_AutoThread;  { one extra variable that can be initializedin order to make the compiler happy }
    AutoThread_count : longint = 0;]

//define[dothread ][(#1);][
begin
    if AutoThread_count>=SImg_maxThreads then cc.error('Internal Error: AutoThread overflow');
    AutoThreads[AutoThread_count]:=C#0.create(#1);
    AutoThread_count+=1;
end;]

//define[threadwait][
AutoThreads[SImg_maxThreads]:=nil;   // makes the compiler happy
while AutoThread_count>0 do
begin
    AutoThread_count-=1;
    AutoThreads[AutoThread_count].destroy;
end;]


{*************************************************************************}
{******* SImg_Halt *******************************************************}
{*************************************************************************}
var SImg_MainTID : longint;

const SImg_Running      : boolean = true;
      SImg_ExitCode     : longint = 0;
      SImg_ExternalExit : boolean = true;

const SImg_ExternalExitCode = 9;

{******* SImg_SIGINT *****************************************************}
procedure SImg_SIGINT(sig:longint); cdecl;
begin
    if SImg_ExternalExit then SImg_ExitCode:=SImg_ExternalExitCode;
    if gettid=SImg_MainTID then
    begin
//ifdef[debug]
        SImg_Info('Received SIGINT at '+SImg_GetStackTrace(get_frame));
//endif
        if SImg_Running then
        begin
            SImg_Running:=false;
            if SImg_Sem_Reset and (SImg_ExitCode<2) then SImg_ExitCode:=2;
            halt(SImg_ExitCode);
        end;
    end
    else begin
        if SImg_Running then fpKill(SImg_MainTID,SIGINT);
    end;
end;

{******* SImg_Halt *******************************************************}
procedure SImg_Halt(ec:longint);
begin
    SImg_ExitCode:=ec;
    SImg_ExternalExit:=false;
    if gettid=SImg_MainTID then halt(ec)
    else begin
        fpKill(SImg_MainTID,SIGINT);
        fpkill(gettid,SIGKILL);
        delay(5000);
    end;
end;

procedure SImg_Halt;
begin
    SImg_Halt(0);
end;


{*************************************************************************}
{******* SImg_SIGUSR2 ****************************************************}
{*************************************************************************}
procedure SImg_SIGUSR2(sig:longint); cdecl;
begin
//ifdef[debug]
    if gettid=SImg_MainTID then SImg_Info('Received SIGUSR2 at'+SImg_GetStackTrace(get_frame));
//endif
end;


{*************************************************************************}
{******* SImg_InitThread *************************************************}
{*************************************************************************}
procedure SImg_InitThread;
var act_int, act_usr1, act_usr2: sigactionrec;

begin
    act_int.sa_handler:=sigactionhandler(@SImg_SIGINT);
    act_int.sa_flags:=0;
{$ifdef linux}
    act_int.sa_restorer:=nil;
{$endif}
    fillchar(act_int.sa_mask,sizeof(act_int.sa_mask),0);

    act_usr1.sa_handler:=sigactionhandler(@SImg_SIGUSR1);
    act_usr1.sa_flags:=0;
{$ifdef linux}
    act_usr1.sa_restorer:=nil;
{$endif}
    fillchar(act_usr1.sa_mask,sizeof(act_usr1.sa_mask),0);

    act_usr2.sa_handler:=sigactionhandler(@SImg_SIGUSR2);
    act_usr2.sa_flags:=0;
{$ifdef linux}
    act_usr2.sa_restorer:=nil;
{$endif}
    fillchar(act_usr2.sa_mask,sizeof(act_usr2.sa_mask),0);

    SImg_MainTid:=gettid;
    fpSigAction(SIGINT, @act_int, nil);
    fpSigAction(SIGUSR1, @act_usr1, nil);
    fpSigAction(SIGUSR2, @act_usr2, nil);
end;

