{  simg -- utilities for the simg package
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

uses dos;

var units,files,exts             : array[0..255] of ansistring;
    files2,exts2                 : array[0..255] of ansistring;
    desc,names                   : array[0..255] of ansistring;
    longdesc,iexts,caps,seealso  : array[0..255] of ansistring;
    dirs,dirs2                   : array[0..255] of ansistring;
    rmlines                      : array[0..255] of longint;

    mu,mf,i,j,k,l,m,line   : longint;
    sr                     : searchrec;
    ns                     : namestr;
    ds,ds2                 : dirstr;
    es                     : extstr;
    s,s2,dest              : ansistring;
    t,u                    : text;
    b                      : boolean;

procedure upstr(var s:ansistring);
var a     : longint;
begin
    for a:=1 to length(s) do
    case s[a] of
        #97..#122 : s[a]:=char(byte(s[a])-32);
        #129      : s[a]:=#154;
        #132      : s[a]:=#142;
        #148      : s[a]:=#153;
    end;
end;

procedure downstr(var s:ansistring);
var a     : longint;
begin
    for a:=1 to length(s) do
    case s[a] of
        #65..#90  : s[a]:=char(byte(s[a])+32);
        #154      : s[a]:=#129;
        #142      : s[a]:=#132;
        #153      : s[a]:=#148;
    end;
end;


procedure replaceref(var s:ansistring);
var i  : longint;
begin
    i:=1;
    while i<length(s)-5 do
    if (s[i]='@') and (copy(s,i,5)='@ref(') then
    begin
        delete(s,i,5);
        while (i<=length(s)) and (s[i]<>')') do
            i+=1;
        if i<=length(s) then delete(s,i,1);
    end
    else i+=1;
end;

procedure replaceref2(var s:ansistring);
var i  : longint;
begin
    i:=1;
    while i<length(s)-5 do
        if (s[i]='@') and (copy(s,i,5)='@ref(') then
        begin
            delete(s,i,5);
            insert('`',s,i);
            while (i<=length(s)) and (s[i]<>')') do
                i+=1;
            if i<=length(s) then delete(s,i,1);
            insert('''''',s,i);
        end
        else i+=1;
end;

procedure rmch32(var s:ansistring);
var i : longint;
begin
    i:=length(s);
    while (i>0) and (s[i]<=#32) do
        i-=1;
    if i<length(s) then delete(s,i+1,length(s)-i);
    i:=1;
    while (i<=length(s)) and (s[i]<=#32) do
        i+=1;
    delete(s,1,i-1);
end;

procedure haltparamerr(s:ansistring);
begin
    writeln(stderr,'usage: pcollect <name of the collection> <name(s) of the collected programs>');
    if s<>'' then writeln(s);
    halt(1);
end;

procedure writelnu(s:ansistring);
var i:longint;
begin
    writeln(u,s{,'  //',line});
    for i:=1 to length(s) do
        if s[i]=#10 then inc(line);
    inc(line);
end;

procedure writelineinfo(fn:ansistring; l:longint);
begin
    writeln(u,'{#line "', fn, '" ',l,'}');
    inc(line);
end;

procedure writelineinfo;
begin
    writelineinfo(dest+'.pas', line+1);
end;

begin
    if paramcount<2 then haltparamerr('');
    dest:=paramstr(1);
    assign(u,dest+'.pas');
    rewrite(u);
    line:=1;
    writelnu('{$I directives.pas}');

    mu:=3;
    units[0]:='cthreads';
    units[1]:='cmem';
    units[2]:='compatun';
    units[3]:='getparam';
    {mu:=1;
    units[0]:='compatun';
    units[1]:='getparam';}

    mf:=-1;
    for i:=2 to paramcount do
    begin
        fsplit(paramstr(i),ds2,ns,es);
        findfirst(paramstr(i),$27,sr);
        while doserror=0 do
        begin
            inc(mf);
            fsplit(sr.name,ds,ns,es);
            files2[mf]:=ns;
            downstr(files2[mf]);
            dirs2[mf]:=ds2;
            exts2[mf]:=es;
            findnext(sr);
        end;
    end;

    i:=0;
    for m:=0 to mf do
    begin
        assign(t,dirs2[m]+files2[m]+exts2[m]);
        files[i]:=files2[m];
        dirs[i]:=dirs2[i];
        exts[i]:=exts2[i];
        desc[i]:='';
        longdesc[i]:='';
        iexts[i]:='';
        caps[i]:='';
        seealso[i]:='';
        names[i]:=files[m];
        reset(t);
        j:=0; k:=0;
        while not eof(t) and (j=0) do
        begin
            readln(t,s); inc(k);
            rmch32(s);

            if copy(s,1,13)='{$description' then
            begin
                delete(s,1,13);
                delete(s,length(s),1);
                rmch32(s);
                desc[i]:=s;
            end;

            if copy(s,1,6)='{$name' then
            begin
                delete(s,1,6);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                names[i]:=s;
            end;

            if copy(s,1,6)='{@name' then   // only one line allowed
            begin
                delete(s,1,6);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                names[i]:=s;
            end;

            if copy(s,1,13)='{@description' then  // only one line allowed
            begin
                delete(s,1,13);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                replaceref(s);
                desc[i]:=''''+s+'''';
            end;

            if copy(s,1,17)='{@longdescription' then
            begin
                delete(s,1,17);
                longdesc[i]:='''';
                rmch32(s);
                b:=length(s)>0;
                while (not b) or (length(s)=0) or (s[length(s)]<>'}') do
                begin
                    replaceref2(s);
                    if b then longdesc[i]+=s+'''#10+'#10+'''';
                    b:=true;
                    readln(t,s); inc(k);
                    l:=length(s);
                    while (l>0) and (s[l]<=#32) do
                        l-=1;
                    if l<length(s) then delete(s,l+1,length(s)-l);
                end;
                replaceref2(s);
                if length(s)=1 then delete(longdesc[i],length(longdesc[i])-5,6)
                else longdesc[i]+=copy(s,1,length(s)-1)+'''';
            end;

            if copy(s,1,9)='{@seealso' then   // only one line allowed
            begin
                delete(s,1,9);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                replaceref(s);
                seealso[i]:=s;
            end;

            if copy(s,1,14)='{@capabilities' then   // only one line allowed
            begin
                delete(s,1,14);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                replaceref(s);
                caps[i]:=s;
            end;

            if copy(s,1,12)='{@extensions' then   // only one line allowed
            begin
                delete(s,1,12);
                rmch32(s);
                delete(s,length(s),1);
                rmch32(s);
                replaceref(s);
                iexts[i]:=s;
            end;

            upstr(s);

            if copy(s,1,4)='USES' then
            begin
                delete(s,1,4);
                rmch32(s);
                while (not eof(t)) and (s[length(s)]<>';') do
                begin
                    readln(t,s2); inc(k);
                    rmch32(s2);
                    upstr(s2);
                    s:=s+s2;
                end;
                j:=k;
                repeat
                    s2:='';
                    l:=1;
                    while (l<=length(s)) and (s[l]<>',') and (s[l]<>';') do
                        l+=1;
                    s2+=copy(s,1,l-1);
                    delete(s,1,l);
                    rmch32(s);
                    downstr(s2);
                    for l:=0 to mu do
                        if units[l]=s2 then s2:='';
                    if s2<>'' then
                    begin
                        inc(mu);
                        units[mu]:=s2;
                    end;
                until length(s)=0;
            end;
        end;
        rmlines[i]:=j;
        close(t);
        if j>0 then inc(i);
    end;
    mf:=i-1;

    if mf<0 then haltparamerr('at least one file is required');

    if mu>=0 then
    begin
        write(u,'uses ');
        for i:=0 to mu-1 do
            write(u,units[i],',');
        writelnu(units[mu]+';');
    end;

    writelnu('');
    writelnu('var paramstr_0 : string;');
    writelnu('');
    writelnu('function paramstr3(i:longint):ansistring;');
    writelnu('begin');
    writelnu('if i=0 then paramstr3:=paramstr_0');
    writelnu('       else paramstr3:=paramstr(i+1);');
    writelnu('end;');
    writelnu('');

    for i:=0 to mf do
    begin
        writelnu('');
        writelnu('procedure '+files[i]+'_mainprocedure;');
        assign(t,dirs[i]+files[i]+'.pas');
        reset(t);
        for j:=1 to rmlines[i] do
            if not eof(t) then readln(t,s);
        s2:='';
        writelineinfo(dirs[i]+files[i]+'.pas', rmlines[i]+1);
        while (not eof(t)) and (s2<>'END.') do
        begin
            readln(t,s);
            s2:=s;
            upstr(s2);
            rmch32(s2);
            if s2<>'END.' then writelnu(s);
        end;
        writelineinfo;
        writelnu('end;');
        writelnu('');
        close(t);
    end;

    writelineinfo;
    writelnu('procedure haltparamerror;');
    writelnu('var s : string;');
    writelnu('    i : longint;');
    writelnu('begin');
    writelnu('writeln(stderr,''usage: ''+paramstr(0)+'' <parameter>'');');
    writelnu('writeln(stderr,''one of the following program parameters is required:'');');
    j:=8;
    for i:=0 to mf do
        if length(names[i])>j then j:=length(names[i]);
    for i:=0 to mf do
    begin
        s:=names[i];
        while length(s)<j do
            s+=#32;
        writelnu('write(stderr,''   '+s+'   '');');
        for k:=1 to length(s) do
            s[k]:=#32;
        writelnu('s:='+desc[i]+';');
        writelnu('for i:=1 to length(s) do');
        writelnu(' if s[i]=#10 then write(stderr, #10,''   '+s+'   '')');
        writelnu('             else write(stderr, s[i]);');
        writelnu('writeln(stderr);');
        writelnu('');
    end;
    writelnu('writeln(stderr);');
    writelnu('writeln(stderr,''Hint: try -h to see the program parameters'');');
    writelnu('writeln(stderr,''Example: simg add       (runs simg add)'');');
    writelnu('writeln(stderr,''         simg add -h    (shows the help)'');');
    writelnu('writeln(stderr);');
    writelnu('halt(1)');
    writelnu('end;');
    writelnu('');
    writelnu('begin');
    writelnu('paramstr2:=@paramstr3;');
    writelnu('paramcount2:=paramcount-1;');
    writelnu('if paramcount<1 then haltparamerror;');
    writelnu('SImg_Config_Default.logFileBase:=paramstr(1);');
    writelnu('SImg_Config_Default.name:=paramstr(1);');
    for i:=0 to mf do
    begin
        if i=0 then writelnu('if paramstr(1)='''+names[i]+''' then')
        else writelnu('   else if paramstr(1)='''+names[i]+''' then');
        writelnu('      begin');
        if longdesc[i]<>'' then writelnu('      longdescription+='+longdesc[i]+'+#10#10;');
        if iexts[i]<>'' then writelnu('      longdescription+=''Supported image extensions: '+iexts[i]+'''#10#10;');
        if caps[i]<>'' then writelnu('      longdescription+=''Capabilities: '+caps[i]+'''#10#10;');
        if seealso[i]<>'' then writelnu('      longdescription+=''See also: '+seealso[i]+'''#10#10;');
        writelnu('      paramstr_0:=paramstr(0)+'' '+names[i]+''';');
        writelnu('      '+files[i]+'_mainprocedure;');
        writelnu('      end');
    end;
    writelnu('   else haltparamerror;');
    writelnu('end.');

    close(u);
end.
