{  simg -- visualization library
   Version 20220317
   Copyright (C) 1999-2022 Stefan Ziegenbalg
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 3 as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see http://www.gnu.org/licenses/.
!}

{$mode objfpc}
unit siv_utils;

interface
uses glib2,gdk2,gdk2pixbuf,gtk2,simage,pango;

procedure siv_print_message(t:longint; const mes:simgstring; print:boolean);
procedure siv_message_quit;

function siv_pixmap_new(var data: array of pchar):PGtkWidget;
function siv_pixmap_button_new(var data: array of pchar):PGtkWidget;
function siv_tooltip_new(parent:PGtkWidget; tip_text:simgstring; tip_private:pchar):PGtkTooltips;
function siv_string_width(parent:PGtkWidget; text:pchar):longint;
procedure siv_widget_set_visible(widget:PGTKWidget; visible:boolean);
function siv_adjustment_get_pos(adj:PGtkAdjustment):double;
procedure siv_adjustment_copy_from(dest,src:PGtkAdjustment);

const siv_prevConfig : CSImg_Config = nil;

implementation
const siv_message_window : PGtkWidget = nil;
      siv_message_clist  : PGtkWidget = nil;


{ ******** siv_message_destroy ******************************************** }
procedure siv_message_destroy(widget:PGtkWidget; pt:pointer); cdecl;
begin
    siv_message_window:=nil;
    siv_message_clist:=nil;
end;

{ ************************************************************************* }
{ ******** siv_print_message ********************************************** }
{ ************************************************************************* }
procedure siv_print_message(t:longint; const mes:simgstring; print:boolean);
var scroll : PGtkWidget;
    row    : longint;
    s      : simgstring;
    pc     : array[0..1] of pchar;

begin
    if siv_prevConfig<>nil then siv_prevConfig.PrintMessage(t,mes,print);
    if not print then exit;
    if (siv_message_window=nil) or (not gtk_is_window(siv_message_window)) or (siv_message_clist=nil) or (not gtk_is_clist(siv_message_clist)) then
    begin
        siv_message_window:=gtk_window_new(GTK_WINDOW_TOPLEVEL);
        s:='Message Window';
        gtk_window_set_title(PGtkWindow(siv_message_window), str2pchar(s));
        g_signal_connect(GTK_OBJECT(siv_message_window), 'destroy', GTK_SIGNAL_FUNC(@siv_message_destroy), nil);
        gtk_widget_show(siv_message_window);
        gdk_window_resize(siv_message_window^.window, 480,128);

        scroll:=gtk_scrolled_window_new(nil,nil);
        gtk_scrolled_window_set_policy(PGtkScrolledWindow(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
        gtk_container_add(PGtkContainer(siv_message_window), scroll);
        gtk_widget_show(scroll);

        siv_message_clist:=gtk_clist_new(1);
        gtk_clist_set_column_auto_resize(PGtkCList(siv_message_clist), 0, true);
        gtk_clist_set_selection_mode(PGtkCList(siv_message_clist), GTK_SELECTION_MULTIPLE);
        gtk_container_add(PGtkContainer(scroll), siv_message_clist);
        gtk_widget_show(siv_message_clist);
    end;
    if t<-3 then s:=''
    else if t=-3 then s:='Info: '
    else if t=-2 then s:='Hint: '
    else if t=-1 then s:='Warning: '
    else if t=0 then s:='Error: '
    else s:='Fatal: ';
    if t>=-1 then writeln(stderr,s+mes);
    s:=s+ansiToUtf8(mes);
    pc[0]:=str2pchar(s); pc[1]:=nil;
    row:=gtk_clist_append(PGtkClist(siv_message_clist), @pc);
    gtk_clist_moveto(PGtkClist(siv_message_clist), row, 0, 1, 0);
end;

{ ************************************************************************* }
{ ******** siv_message_quit *********************************************** }
{ ************************************************************************* }
procedure siv_message_quit;
begin
if (siv_message_window<>nil) and gtk_is_window(siv_message_window)
   and (siv_message_clist<>nil) and gtk_is_clist(siv_message_clist)
   then gtk_widget_destroy(siv_message_window);
end;


{ ************************************************************************* }
{ ******** siv_pixmap_new ************************************************* }
{ ************************************************************************* }
function siv_pixmap_new(var data: array of pchar):PGtkWidget;
var pixmap           : PGdkPixbuf;
    widget           : PGtkWidget;
begin
    pixmap:=gdk_pixbuf_new_from_xpm_data(@data);
    widget:=gtk_image_new_from_pixbuf(pixmap);
    result:=widget;
end;


{ ************************************************************************* }
{ ******** siv_pixmap_button_new ****************************************** }
{ ************************************************************************* }
function siv_pixmap_button_new(var data: array of pchar):PGtkWidget;
var pixmapwid,button : PGtkWidget;
begin
    pixmapwid:=siv_pixmap_new(data);
    gtk_widget_show(pixmapwid);
    button:=gtk_button_new;
    gtk_button_set_relief(PGtkButton(button),gtk_relief_none);
    gtk_container_add(PGtkContainer(button),pixmapwid);
    result:=button;
end;


{ ************************************************************************* }
{ ******** siv_tooltip_new ************************************************ }
{ ************************************************************************* }
function siv_tooltip_new(parent:PGtkWidget; tip_text:simgstring; tip_private:pchar):PGtkTooltips;
var widget  : PGtkTooltips;
begin
    widget:=gtk_tooltips_new;
    gtk_tooltips_set_tip(widget,parent,str2pchar(tip_text),tip_private);
    result:=widget;
end;


{ ************************************************************************* }
{ ******** siv_string_width *********************************************** }
{ ************************************************************************* }
function siv_string_width(parent:PGtkWidget; text:pchar):longint;
var layout : PPangoLayout;
begin
    layout:=gtk_widget_create_pango_layout(parent,text);
    pango_layout_get_pixel_size(layout, @result, nil);
    g_object_unref(layout);
end;

{ ************************************************************************* }
{ ******** siv_widget_set_visible ***************************************** }
{ ************************************************************************* }
procedure siv_widget_set_visible(widget:PGTKWidget; visible:boolean);
begin
    if GTK_WIDGET_VISIBLE(widget)<>visible then
    begin
        if visible then gtk_widget_show(widget)
        else gtk_widget_hide(widget);
    end;
end;

{ ************************************************************************* }
{ ******** siv_adjustment_get_pos ***************************************** }
{ ************************************************************************* }
function siv_adjustment_get_pos(adj:PGtkAdjustment):double;
begin
    if (adj=nil) or (adj^.upper-adj^.lower<adj^.page_size) then result:=0.5
    else result:=adj^.value/(adj^.upper-adj^.page_size-adj^.lower+1);
end;

{ ************************************************************************* }
{ ******** siv_adjustment_copy ******************************************** }
{ ************************************************************************* }
procedure siv_adjustment_copy_from(dest,src:PGtkAdjustment);
begin
    dest^.value:=src^.value;
    dest^.lower:=src^.lower;
    dest^.upper:=src^.upper;
    dest^.step_increment:=src^.step_increment;
    dest^.page_increment:=src^.page_increment;
    dest^.page_size:=src^.page_size;
    gtk_adjustment_changed(dest);
    gtk_adjustment_value_changed(dest);
end;

end.
